import typing as t
import warnings

from atproto_client import models


class _BackwardCompatibility:
    @staticmethod
    def _strong_ref_arg_backward_compatibility(
        uri: t.Optional[str] = None,
        cid: t.Optional[str] = None,
        subject: t.Optional[models.ComAtprotoRepoStrongRef.Main] = None,
    ) -> models.ComAtprotoRepoStrongRef.Main:
        """Temporary method for backward compatibility."""
        if subject:
            subject_obj = subject
            warnings.warn(
                'The `subject` argument is deprecated and will be removed soon!.',
                DeprecationWarning,
                stacklevel=2,
            )
        elif isinstance(uri, models.ComAtprotoRepoStrongRef.Main):
            warnings.warn(
                'The `uri` argument should be a string. Not ComAtprotoRepoStrongRef!', DeprecationWarning, stacklevel=2
            )
            subject_obj = uri
        else:
            if not cid or not uri:
                raise ValueError('URI and CID must be provided. It is temporary optional for backward compatibility.')

            subject_obj = models.ComAtprotoRepoStrongRef.Main(cid=cid, uri=uri)

        return subject_obj



import typing as t

from atproto_client.client.base import AsyncClientBase
from atproto_client.namespaces import async_ns

# TODO(MarshalX): This file should be autogenerated!


class AsyncClientRaw(AsyncClientBase):
    """Group all root namespaces."""

    com: 'async_ns.ComNamespace'
    app: 'async_ns.AppNamespace'

    def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
        super().__init__(*args, **kwargs)

        self.com = async_ns.ComNamespace(self)
        self.app = async_ns.AppNamespace(self)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.feed.getFeedGenerator`."""

    feed: str  #: AT-URI of the feed generator record.


class ParamsDict(te.TypedDict):
    feed: str  #: AT-URI of the feed generator record.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.feed.getFeedGenerator`."""

    is_online: (
        bool
    )  #: Indicates whether the feed generator service has been online recently, or else seems to be inactive.
    is_valid: bool  #: Indicates whether the feed generator service is compatible with the record declaration.
    view: 'models.AppBskyFeedDefs.GeneratorView'  #: View.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`app.bsky.notification.registerPush`."""

    app_id: str  #: App id.
    platform: str  #: Platform.
    service_did: str  #: Service did.
    token: str  #: Token.


class DataDict(te.TypedDict):
    app_id: str  #: App id.
    platform: str  #: Platform.
    service_did: str  #: Service did.
    token: str  #: Token.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.searchActorsSkeleton`."""

    q: str  #: Search query string; syntax, phrase, boolean, and faceting is unspecified, but Lucene query syntax is recommended. For typeahead search, only simple term match is supported, not full syntax.
    cursor: t.Optional[
        str
    ] = None  #: Optional pagination mechanism; may not necessarily allow scrolling through entire result set.
    limit: t.Optional[int] = Field(default=25, ge=1, le=100)  #: Limit.
    typeahead: t.Optional[bool] = None  #: If true, acts as fast/simple 'typeahead' query.


class ParamsDict(te.TypedDict):
    q: str  #: Search query string; syntax, phrase, boolean, and faceting is unspecified, but Lucene query syntax is recommended. For typeahead search, only simple term match is supported, not full syntax.
    cursor: te.NotRequired[
        t.Optional[str]
    ]  #: Optional pagination mechanism; may not necessarily allow scrolling through entire result set.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    typeahead: te.NotRequired[t.Optional[bool]]  #: If true, acts as fast/simple 'typeahead' query.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.searchActorsSkeleton`."""

    actors: t.List['models.AppBskyUnspeccedDefs.SkeletonSearchActor']  #: Actors.
    cursor: t.Optional[str] = None  #: Cursor.
    hits_total: t.Optional[
        int
    ] = None  #: Count of search hits. Optional, may be rounded/truncated, and may not be possible to paginate through all hits.



from atproto_crypto.algs import ALGORITHM_TO_CLASS
from atproto_crypto.did import parse_did_key
from atproto_crypto.exceptions import UnsupportedSignatureAlgorithmError


def verify_signature(did_key: str, signing_input: bytes, signature: bytes) -> bool:
    """Verify signature.

    Args:
        did_key: DID key.
        signing_input: Signing input (data).
        signature: Signature.

    Returns:
        bool: True if signature is valid, False otherwise.
    """
    parsed_did_key = parse_did_key(did_key)
    if parsed_did_key.jwt_alg not in ALGORITHM_TO_CLASS:
        raise UnsupportedSignatureAlgorithmError('Unsupported signature alg')

    algorithm_class = ALGORITHM_TO_CLASS[parsed_did_key.jwt_alg]
    return algorithm_class().verify_signature(parsed_did_key.key_bytes, signing_input, signature)



from atproto_core.exceptions import AtProtocolError


class DidPlcResolverError(AtProtocolError):
    ...


class DidWebResolverError(AtProtocolError):
    ...


class PoorlyFormattedDidError(AtProtocolError):
    ...


class UnsupportedDidWebPathError(AtProtocolError):
    ...


class UnsupportedDidMethodError(AtProtocolError):
    ...


class PoorlyFormattedDidDocumentError(AtProtocolError):
    ...


class DidNotFoundError(AtProtocolError):
    ...


class AtprotoDataParseError(AtProtocolError):
    ...



import typing as t

from atproto_core.exceptions import AtProtocolError

if t.TYPE_CHECKING:
    from atproto_client.request import Response


_DEFAULT_LOGING_REQUIRED_ERROR_MESSAGE = 'To perform this action, you must be logged in. Use the `login` method first.'


class ModelError(AtProtocolError):
    ...


class ModelFieldNotFoundError(ModelError):
    ...


class RequestErrorBase(AtProtocolError):
    def __init__(self, response: t.Optional['Response'] = None) -> None:
        self.response: t.Optional['Response'] = response


class NetworkError(RequestErrorBase):
    ...


class InvokeTimeoutError(NetworkError):
    ...


class UnauthorizedError(RequestErrorBase):
    ...


class RequestException(RequestErrorBase):
    ...


class BadRequestError(RequestErrorBase):
    ...


class LoginRequiredError(AtProtocolError):
    def __init__(self, message: t.Optional[str] = _DEFAULT_LOGING_REQUIRED_ERROR_MESSAGE) -> None:
        super().__init__(message)



import typing as t

import httpx

from atproto_identity.did.resolvers.base_resolver import AsyncBaseResolver, BaseResolver
from atproto_identity.exceptions import DidPlcResolverError

if t.TYPE_CHECKING:
    from atproto_identity.cache.base_cache import AsyncDidBaseCache, DidBaseCache


class DidPlcResolver(BaseResolver):
    def __init__(
        self,
        plc_url: t.Optional[str] = None,
        timeout: t.Optional[float] = None,
        cache: t.Optional['DidBaseCache'] = None,
    ) -> None:
        super().__init__(cache)

        self._plc_url = plc_url
        self._timeout = timeout

        self._client = httpx.Client()

    def resolve_without_validation(self, did: str) -> t.Optional[t.Dict[str, t.Any]]:
        try:
            response = self._client.get(f'{self._plc_url}/{did}', timeout=self._timeout)

            if response.status_code == 404:
                return None

            response.raise_for_status()
            return response.json()
        except httpx.HTTPError as e:
            raise DidPlcResolverError(f'Error resolving DID {did}') from e


class AsyncDidPlcResolver(AsyncBaseResolver):
    def __init__(
        self,
        plc_url: t.Optional[str] = None,
        timeout: t.Optional[float] = None,
        cache: t.Optional['AsyncDidBaseCache'] = None,
    ) -> None:
        super().__init__(cache)

        self._plc_url = plc_url
        self._timeout = timeout

        self._client = httpx.AsyncClient()

    async def resolve_without_validation(self, did: str) -> t.Optional[t.Dict[str, t.Any]]:
        try:
            response = await self._client.get(f'{self._plc_url}/{did}', timeout=self._timeout)

            if response.status_code == 404:
                return None

            response.raise_for_status()
            return response.json()
        except httpx.HTTPError as e:
            raise DidPlcResolverError(f'Error resolving DID {did}') from e



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.recordWithMedia`."""

    media: te.Annotated[
        t.Union['models.AppBskyEmbedImages.Main', 'models.AppBskyEmbedExternal.Main'], Field(discriminator='py_type')
    ]  #: Media.
    record: 'models.AppBskyEmbedRecord.Main'  #: Record.

    py_type: te.Literal['app.bsky.embed.recordWithMedia'] = Field(
        default='app.bsky.embed.recordWithMedia', alias='$type', frozen=True
    )


class View(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.recordWithMedia`."""

    media: te.Annotated[
        t.Union['models.AppBskyEmbedImages.View', 'models.AppBskyEmbedExternal.View'], Field(discriminator='py_type')
    ]  #: Media.
    record: 'models.AppBskyEmbedRecord.View'  #: Record.

    py_type: te.Literal['app.bsky.embed.recordWithMedia#view'] = Field(
        default='app.bsky.embed.recordWithMedia#view', alias='$type', frozen=True
    )
